#include "Battle.h"
#include "DrawWindow.h"
#include "Combatant.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include "SDL_ttf.h"

#include <iostream>
#include <ostream>
#include <sstream>
using namespace std;

#include "InputObj.h"
#include "UserInput.h"
#include "CpuInput.h"
#include <stdlib.h>

#define BattleStateWaitingForInit 0
#define BattleStateBattleBegun 1
#define BattleStateChooseAttack 2
#define BattleStatePlayerAttackBegin 3
#define BattleStateEnemyAttackBegin 4
#define BattleStateRoundOver 5
#define BattleStateBattleOver 6
#define BattleStateYouLose 7

Battle :: Battle ()
{
  battleState = BattleStateWaitingForInit;
  battleTimer = 0;
  numVicts = 0;
  hpBack = 0;
  overkillBonus = 0;
  countVict = false;
  leftCombatant = NULL;
  rightCombatant = NULL;
}

void Battle :: reset ()
{
  //clears all relevant data, ready to restart game.
  battleState = BattleStateWaitingForInit;
  battleTimer = 0;
  numVicts = 0;
  hpBack = 0;
  overkillBonus = 0;
  countVict = false;
  leftCombatant = NULL;
  rightCombatant = NULL;
}

void Battle :: init (Combatant * lComb, Combatant * rComb, SDL_Color & rulz)
{
  //gets the involved parties, loads stats.
  //update parameters. main calls this at the beginning of a battle.

  endurance = 1000;
  accuracy = 25;
  battleState = BattleStateBattleBegun;
  battleTimer = 0;

  leftCombatant = lComb;
  rightCombatant = rComb;
  leftTurn = true;
  numHits = 0;
  numDmg = 0;
  hpBack = 0;
  overkillBonus = 0;

  drawRules.r = rulz.r;
  drawRules.g = rulz.g;
  drawRules.b = rulz.b;
  drawRules.unused = rulz.unused;

  if (dw == NULL)
    dw = new DrawWindow ();
  dw -> init (drawRules);
}

void Battle :: render (SDL_Surface * dest, TTF_Font * font)
{
  string s;

  SDL_Color textCol;
  textCol.r = 255;
  textCol.g = 255;
  textCol.b = 255;

  SDL_Color textCol2;
  textCol2.r = 64;
  textCol2.g = 64;
  textCol2.b = 64;

  SDL_Rect textPosa;
  textPosa.x = 0;
  textPosa.y = 0;

  SDL_Rect textPos1;
  textPos1.x = 170;
  textPos1.y = 350;

  SDL_Rect textPos2;
  textPos2.x = 170;
  textPos2.y = 369;

  SDL_FillRect(dest, NULL, SDL_MapRGB(dest -> format, 0, 0, 0));

  s.append ("Escape = Quit, F10 = restart");
  textOutput = TTF_RenderText_Solid (font, s.c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);
  SDL_FillRect(textOutput, NULL, SDL_MapRGB(dest -> format, 0, 0, 0));

  //render based on the BattleState
  if (battleState == BattleStatePlayerAttackBegin || battleState == BattleStateEnemyAttackBegin || battleState == BattleStateRoundOver || battleState == BattleStateBattleOver || battleState == BattleStateChooseAttack || battleState == BattleStateBattleBegun)
    renderCombatants (dest, font);

  if (battleState == BattleStateBattleBegun)
    renderBattleIntro (dest, font, textCol, textPos1);

  if (battleState == BattleStateBattleOver)
    renderBattleExtro (dest, font, textCol, textPos1);

  if (battleState == BattleStatePlayerAttackBegin)
    renderEndurance (dest, font, textCol, textCol2, textPos1);

  if (battleState == BattleStateRoundOver)
    renderResults (dest, font, textCol, textPos1);

  if (battleState == BattleStatePlayerAttackBegin || battleState == BattleStateRoundOver)
    {
      if (leftTurn)
	dw -> render (dest, leftTurn);

      //draw the "mouse"
      if (leftTurn && leftCombatant -> getIo () != NULL)
	leftCombatant -> getIo () -> render (dest, accuracy);
    }
}

void Battle :: renderCombatants (SDL_Surface * dest, TTF_Font * font)
{
  //players should draw
  leftCombatant -> render (dest, font); 
  rightCombatant -> render (dest, font); 

  SDL_Rect tPos;
  tPos.x = 30;
  tPos.y = 90;

  SDL_Color textCol;
  textCol.r = 192;
  textCol.g = 192;
  textCol.b = 192;

  ostringstream oss (ostringstream::out);
  oss << Uint32(numVicts) << " victories so far...";

  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &tPos);
}

void Battle :: renderBattleIntro (SDL_Surface * dest, TTF_Font * font, SDL_Color & textCol, SDL_Rect & textPos1)
{
  string s;
  if (rightCombatant != NULL)
    s.append (rightCombatant -> getName ());

  s.append (" approaches!");
  textOutput = TTF_RenderText_Solid (font, s.c_str(), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
}

void Battle :: renderBattleExtro (SDL_Surface * dest, TTF_Font * font, SDL_Color & textCol, SDL_Rect & textPos1)
{
  ostringstream oss (ostringstream::out);
  ostringstream oss2 (ostringstream::out);

  if (leftCombatant -> getAreYouDead ())
    oss << "You Lose. " << numVicts << " victories total.";
  else
    oss << "You win! " << numVicts << " victories.";

  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos1);  

  textPos1.y += 15;
  if (!leftCombatant -> getAreYouDead ())
    {
      oss2 << "You gain " << (Uint32(hpBack) + Uint32(overkillBonus));

      if (overkillBonus > 0)
	oss2 << " (includes +" << Uint32(overkillBonus) << " overkill bonus)";

      oss2 << " HP.";

      textOutput = TTF_RenderText_Solid (font, oss2.str ().c_str (), textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);  
    }
  textPos1.y -= 15;
}

void Battle :: renderEndurance (SDL_Surface * dest, TTF_Font * font, SDL_Color & textCol, SDL_Color & textCol2, SDL_Rect & textPos1)
{
  string s;

  SDL_Rect endFill;
  endFill.x = 170;
  endFill.y = 365;
  endFill.w = 300;
  endFill.h = 15;

  ostringstream oss (ostringstream::out);

  //draw based on battle state.
  if (dw -> getDrawingStatus () == 0)
    {
      s.append ("Connect the dots by clicking and dragging the mouse.");
      textOutput = TTF_RenderText_Solid (font, s.c_str (), textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
    }
  if (dw -> getDrawingStatus () == 1)
    {
      endFill.w = 300 * getEndurance() / 1000;

      if (getEndurance() < 0 || getEndurance() > 1000)
	endFill.w = 300;

      textPos1.y += 15;

      if (getEndurance() > 250)
	SDL_FillRect(dest, &endFill, SDL_MapRGB(dest -> format, 32, 32, 32));
      else
	SDL_FillRect(dest, &endFill, SDL_MapRGB(dest -> format, 160, 160, 160));
      s.append ("Endurance");
      textOutput = TTF_RenderText_Solid (font, s.c_str(), textCol2);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);

      textPos1.y -= 15;

      oss << "Hits: " << numHits;

      if (overkillBonus > 0)
	oss << "( " << Uint32(overkillBonus) << " overkill)";

      textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
      SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
    }
}

void Battle :: renderResults (SDL_Surface * dest, TTF_Font * font, SDL_Color & textCol, SDL_Rect & textPos1)
{
  ostringstream oss (ostringstream::out);
  oss << (numHits * numDmg) << " Damage Total";

  if (overkillBonus > 0)
    oss << ", with overkill of " << Uint32(overkillBonus) << " Hits.";

  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPos1);
}

void Battle :: gameLoop ()
{
  Combatant * comb;
  Uint16 targetCode = 0; //based on attack
  CpuInput * cpu = NULL;

  if (leftTurn)
    {
      comb = leftCombatant;

      //attack should load this...
      targetCode += 1; //leftCombatant attacks
      targetCode += 2; //rightCombatant defends
    }
  else
    {
      comb = rightCombatant;
    }

  //battleState
  //0: Waiting for initialization
  //1: Announce that battle has begun!
  //2: In battle. Draw, attack, deal damage. (May have to expand this.)
  //3: Round over. Show results of the round. Go back to step 2.
  //4: The fight is over, show results.
  switch (battleState){
  case BattleStateWaitingForInit:
    break;
  case BattleStateBattleBegun:
    battleTimer ++;
    if (battleTimer > 30)
      {battleTimer = 0; battleState = BattleStatePlayerAttackBegin;}
    break;
  case BattleStateChooseAttack:
    numHits = 0;
    numDmg = 0;

    //this is when you should choose your attack.
    if (leftTurn)
      battleState = BattleStatePlayerAttackBegin;
    else
      battleState = BattleStateEnemyAttackBegin;
    break;
  case BattleStatePlayerAttackBegin:
    //if it's the AI's turn, tell it to do it's AI thing.
    cpu = dynamic_cast <CpuInput *> (comb -> getIo ());
    if (cpu != NULL)
      cpu -> getNextMove ();      

    if (comb -> getIo () != NULL){
    dw -> drawEvaluator (comb -> getIo () -> getButton1 (), accuracy, 
			 comb -> getIo () -> getCursorPosX () - dw -> getDimensions().x, 
			 comb -> getIo () -> getCursorPosY () - dw -> getDimensions().y, 
			 endurance, targetCode);
    }

    if (dw -> getDrawingStatus () == 2)
      {
	battleTimer = 0;
	battleState = BattleStateRoundOver;
      }
    break;
  case BattleStateEnemyAttackBegin:
    //if it's the AI's turn, tell it to do it's AI thing.
    cpu = dynamic_cast <CpuInput *> (comb -> getIo ());
    if (cpu != NULL)
      {
	cpu -> getNextMove ();
	if (cpu -> getMoveDone ())
	  battleState = BattleStateRoundOver;
      }
    else
	battleState = BattleStateRoundOver;
    battleTimer = 0;
    break;
  case BattleStateRoundOver:
    battleTimer ++;
    if (battleTimer > 30)
      {
	//check to see if the battle is over.
	if (rightCombatant -> getAreYouDead () || leftCombatant -> getAreYouDead ())
	  {
	    battleState = BattleStateBattleOver;
	  }
	else
	  {
	    //otherwise, Switch turns, the round is over.
	    leftTurn = !leftTurn;
	    endurance = 1000;
	    battleState = BattleStateChooseAttack;
	    comb -> getIo () -> clearInput ();
	    dw -> init (drawRules);
	  }
      }
    break;
  case BattleStateBattleOver:
    battleTimer ++;

    if (!countVict)
      {
	if (rightCombatant -> getAreYouDead ())
	  {
	    numVicts ++;

	    hpBack += 1 + (getNumberOfVictories ()/6);

	    //figure out better overkill mechanism...
	    if (getNumberOfVictories () < 6)
	      overkillBonus = overkillBonus / 10;
	    else if (getNumberOfVictories () < 12)
	      overkillBonus = overkillBonus / 8;
	    else
	      overkillBonus = overkillBonus / 6;
	    
	    //gain hp for killing the enemy
	    leftCombatant -> addHP (hpBack);
	    leftCombatant -> addHP (overkillBonus);
	    
	    countVict = true;
	  }
      }

    if (rightCombatant -> getAreYouDead () && battleTimer > 60)
      {
	countVict = false;
	battleState = BattleStateWaitingForInit;
	battleTimer = 0;
      }
    else if (leftCombatant -> getAreYouDead () && battleTimer > 150)
      {
	battleState = BattleStateYouLose;
	battleTimer = 0;
      }  
    break;
  default:
    break;
  }
}

void Battle :: processInput (const SDL_Event & mainEvent)
{
  //give control to whichever player is in action

  Combatant * cInUse;
  Uint16 targetC;

  if (leftTurn)
    cInUse = leftCombatant;
  else
    cInUse = rightCombatant;

  if (cInUse != NULL)
    cInUse -> processInput (mainEvent);

  switch(mainEvent.type){
  case SDL_USEREVENT:
    //read the struct
    targetC = ((SDL_Rect *) (mainEvent.user.data1)) -> w;

    numHits ++;

    //calculate damage here
    numDmg = ((SDL_Rect *) (mainEvent.user.data1)) -> h;

    if ((targetC & 1) > 0)
      {
	leftCombatant -> animate (1, 3);
      }
    else
      {
	rightCombatant -> animate (1, 3);
      }

    //deal damage to target
    if ((targetC & 2) > 0)
      {
	//overkill bonus
	if (rightCombatant -> takeDamage (numDmg))
	  overkillBonus ++;
      }
    else
      leftCombatant -> takeDamage (numDmg);
    break;
  default:
    break;
  }
}

Uint8 Battle :: getStatus ()
{ return battleState; }

Sint32 Battle :: getEndurance ()
{ return endurance; }

Uint32 Battle :: getNumberOfVictories ()
{  return numVicts; }

Battle :: ~Battle ()
{
}
