#ifndef BATTLE_H
#define BATTLE_H

#include "SDL.h"

#include "DrawWindow.h"
#include "InputObj.h"
#include "UserInput.h"
#include "SDL_ttf.h"
#include "Combatant.h"

//Battle is the part of the game that handles the line-drawing conflict.
//Main sends the necessary information (who is involved, scenario, etc etc.)
//Battle processes the fight, and updates the end result.
//Main calls on Battle for end fight stats, and then reupdates it.

class Battle {
 public:
  Battle ();

  void reset ();
  void init (Combatant *, Combatant *, SDL_Color &);

  void render (SDL_Surface *, TTF_Font *);
  void renderCombatants (SDL_Surface *, TTF_Font *);
  void renderBattleIntro (SDL_Surface *, TTF_Font *, SDL_Color &, SDL_Rect &);
  void renderBattleExtro (SDL_Surface *, TTF_Font *, SDL_Color &, SDL_Rect &);
  void renderEndurance (SDL_Surface *, TTF_Font *, SDL_Color &, SDL_Color &, SDL_Rect &);
  void renderResults (SDL_Surface *, TTF_Font *, SDL_Color &, SDL_Rect &);

  void gameLoop ();
  void processInput (const SDL_Event & mainEvent);

  Sint32 getEndurance ();
  Uint8 getStatus ();
  Uint32 getNumberOfVictories ();

  ~Battle ();
 protected:
  SDL_Surface * textOutput;
  
  Uint8 battleTimer;
  Uint8 battleState;

  DrawWindow * dw;

  Sint32 endurance;
  Uint8 accuracy;

  bool leftTurn;
  Combatant * leftCombatant;
  Combatant * rightCombatant;
  SDL_Color drawRules;

  Uint8 hpBack;
  Uint8 overkillBonus;
  Uint16 numHits;
  Uint16 numDmg;
  Uint32 numVicts;
  bool countVict;
};

#endif
