#include "Combatant.h"

#include "SDL.h"
#include "SDL_ttf.h"
#include "SDL_gfxPrimitives.h"

#include <vector>
#include <string>
#include <iostream>
#include <ostream>
#include <sstream>
using namespace std;

#include "InputObj.h"

Combatant :: Combatant ()
{
  name = new string ("Blank");
  maxHp = 20;
  hp = 20;
  playerSide = true;
  areYouDead = false;
  io = NULL;
  animMode = 0;
  animTimer = 0;
}

void Combatant :: setStats (Uint8 maxHP, string & n)
{
  //should be called for each new enemy.
  maxHp = maxHP;
  hp = maxHp;

  name = new string (n);
}

void Combatant :: setInput (InputObj * i)
{ io = i; }

void Combatant :: setSide (bool b)
{ playerSide = b; }

void Combatant :: processInput (const SDL_Event & mainEvent)
{
  if (io != NULL)
    io -> processInput (mainEvent);
}

void Combatant :: render (SDL_Surface * dest, TTF_Font * font)
{
  //draw character
  //NOTE: Waiting for impending animation routines...
  Uint32 height = 100; //get the height
  Uint32 width = 100; //get the width

  SDL_Rect picPos;

  if (playerSide)
    picPos.x = 85 - (width/2);
  else
    picPos.x = 555 - (width/2);

  picPos.y = 330 - height;
  picPos.w = width;
  picPos.h = height;

  //draw the character (should be changed once animation is decided.)
  Uint32 col;

  if (animMode == 0)
    col = SDL_MapRGBA(dest -> format, 128, 128, 128, 255);
  else if (animMode == 1) //attacking
    col = SDL_MapRGBA(dest -> format, 192, 192, 192, 255);
  else if (animMode == 2) //got hit
    col = SDL_MapRGBA(dest -> format, 128, 0, 0, 255);
  else //dead
    col = SDL_MapRGBA(dest -> format, 64, 64, 64, 255);

  if (animMode != 0 || animMode != 3)
    {
      if (animTimer == 0)
	animMode = 0;
      animTimer --;
    }

  SDL_FillRect(dest, &picPos, col);

  //and now, for text.
  SDL_Color textCol;
  textCol.r = 255;
  textCol.g = 255;
  textCol.b = 255;

  SDL_Rect textPosa;
  if (playerSide)
    textPosa.x = 20;
  else
    textPosa.x = 490;
  textPosa.y = 50;
  textOutput = TTF_RenderText_Solid (font, name -> c_str(), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);

  //draw hp bar here
  SDL_Rect hpBarDim;
  if (playerSide)
    hpBarDim.x = 10;
  else
    hpBarDim.x = 480;

  hpBarDim.y = 70;
  hpBarDim.w = 150;
  hpBarDim.h = 20;

  SDL_Color hpBarCol;
  hpBarCol.r = 96;
  hpBarCol.g = 32;
  hpBarCol.b = 32;

  SDL_FillRect(dest, &hpBarDim, SDL_MapRGBA(dest -> format, 32, 32, 32, 230));

  if (playerSide)
    hpBarDim.x = 10;
  else
    hpBarDim.x = 380;
  hpBarDim.w = getHp () * 150 / getMaxHp ();

  if (!playerSide)
    hpBarDim.x = 630 - hpBarDim.w;

  SDL_FillRect(dest, &hpBarDim, SDL_MapRGBA(dest -> format, 96, 32, 32, 255));

  hpBarDim.w = 150;

  if (!playerSide)
    hpBarDim.x = 630 - hpBarDim.w;

  hpBarCol.r = getHp () * 255 / getMaxHp ();
  rectangleRGBA(dest, hpBarDim.x, hpBarDim.y,
		hpBarDim.x + hpBarDim.w, hpBarDim.y + hpBarDim.h,
		hpBarCol.r, hpBarCol.r, hpBarCol.r, 255);

  if (!playerSide)
    hpBarDim.x = 610;

  textPosa.y = 70;

  ostringstream oss (ostringstream::out);
  oss << int(getHp ());
  textOutput = TTF_RenderText_Solid (font, oss.str ().c_str (), textCol);
  SDL_BlitSurface(textOutput, NULL, dest, &textPosa);
}

bool Combatant :: takeDamage (Uint8 dmg)
{
  bool overkill = false;

  if (dmg > hp)
    overkill = true;

  if (dmg >= hp)
    {
      hp = 0; areYouDead = true;
      animate (3, 30);
    }
  else
    {
      animate (2, 5);
      hp -= dmg;
    }
  return overkill;
}

Uint8 Combatant :: getHp ()
{ return hp; }

Uint8 Combatant :: getMaxHp ()
{ return maxHp; }

InputObj * Combatant :: getIo ()
{ return io; }

bool Combatant :: getAreYouDead ()
{
  return areYouDead;
}

void Combatant :: animate (Uint8 index, Uint32 time)
{
  //put real animation info here
  animMode = index;
  animTimer = time;
}

string Combatant :: getName ()
{
  return *(new string (*name));
}

void Combatant :: setName (string newName)
{
  name -> assign (newName);
}

void Combatant :: addMaxHP (Uint8 mp)
{
  if (maxHp + mp > 256)
    maxHp = 256;
  else
    maxHp += mp;

  hp = maxHp;
}

void Combatant :: setMaxHP (Uint8 mp)
{
  maxHp = mp;
  if (hp > maxHp)
    hp = maxHp;
}

void Combatant :: addHP (Uint8 h)
{
  if (h + hp > maxHp)
    hp = maxHp;
  else
    hp += h;
}

Combatant :: ~Combatant ()
{
}
