#include "DrawWindow.h"
#include "LinePattern.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"

#include <iostream>
using namespace std;

DrawWindow :: DrawWindow ()
{
  dimensions.x = 170;
  dimensions.y = 30;
  dimensions.w = 300;
  dimensions.h = 300;
  lp = new LinePattern;
  drawingState = 0;
  timeFromLast = 0;
  attStruct = NULL;
}

void DrawWindow :: init (SDL_Color & rulz)
{
  if (lp != NULL)
    lp -> init (dimensions, rulz);

  endBurn = rulz.unused;

  drawingState = 0;
  timeFromLast = 0;
  numHits = 0;
}

void DrawWindow :: render (SDL_Surface * dest, bool col)
{
  if (col)
    SDL_FillRect(dest, &dimensions, SDL_MapRGBA(dest -> format, 0, 0, 128, 230));
  else
    SDL_FillRect(dest, &dimensions, SDL_MapRGBA(dest -> format, 128, 0, 0, 230));

  if (lp != NULL)
    lp -> render (dest, dimensions);
}

void DrawWindow :: setLinePattern (LinePattern * llp)
{
  lp = llp;
}

void DrawWindow :: drawEvaluator (bool button1, Uint8 accuracy, Sint16 posX, Sint16 posY, Sint32 & end, Uint16 targetCode)
{
  //figures out how to handle the mouse's position in relation to the line.

  SDL_Rect contactPoint;
  int numPoints;

  SDL_Event attMade;

  //drawingState keeps track of the status:
  //0: Has not started. Waiting for user to click.
  //1: User has clicked, Has started evaluating.
  //2: User has released/out of endurance. No longer evaluating.

  if (drawingState == 0 && !button1)
    return;

  if (drawingState == 2)
    return;

  if (drawingState == 1 && (!button1 || end <= 0))
    {
      drawingState = 2;
      return;
    }

  if (drawingState == 0 && button1)
    drawingState = 1;

  //OK, evaluate.

  numPoints = lp -> getContactPoint (contactPoint);

  int dx = contactPoint.x - posX;
  int dy = contactPoint.y - posY;

  Uint32 distSq = (dx * dx) + (dy * dy);
  Uint32 totalAcc = accuracy + contactPoint.w;

  Sint32 endToLose = 0;

  timeFromLast ++;

  //if mouse is at (or close enough to endPoint...)
  if (distSq <= (totalAcc * totalAcc))
    {
      //tell the line pattern to get the next point.
      lp -> getNewLine (dimensions, false);

      attStruct = new SDL_Rect ();
      attStruct -> w = targetCode;
      attStruct -> h = 1; //damage

      attMade.type = SDL_USEREVENT;
      attMade.user.code = 0;
      attMade.user.data1 = (void *) attStruct;
      attMade.user.data2 = 0;
      SDL_PushEvent(&attMade);

      numHits++;

      //reset timer from the last draw.
      timeFromLast = 0;
    }

  //reduce endurance here.  
  //punish for taking too long.
  if (timeFromLast > 3)
    {
      switch (endBurn){
      case 0: endToLose += 40; break;
      case 1: endToLose += 50; break;
      case 2: endToLose += 66; break;
      case 3:
      default: endToLose += 80; break;
      }
    }
  end -= endToLose;
}

SDL_Rect DrawWindow :: getDimensions ()
{
  SDL_Rect * tempSDL = new SDL_Rect (dimensions);
  return (*tempSDL);
}

int DrawWindow :: getDrawingStatus ()
{ return drawingState; }

DrawWindow :: ~DrawWindow ()
{
}
