#include <math.h>
#include "LinePattern.h"

#include "SDL.h"
#include "SDL_gfxPrimitives.h"
#include <vector>
#include <list>
#include <iostream>
using namespace std;

//I'm using SDL_Rect as the base structure for the points you're supposed to connect. x, y are the positions.

LinePattern :: LinePattern ()
{}

void LinePattern :: init (SDL_Rect dim, SDL_Color & rulz)
{
  //restarts the pattern, given pattern rules.
  points.resize (0);

  //randomize drawing stuff
  ang1 = rand () % 360;
  ang2 = ang1 + 180;
  if (ang2 > 360)
    ang2 -= 360;

  sin1 = sin (ang1);
  cos1 = cos (ang2);
      
  sin2 = sin (ang2);
  cos2 = cos (ang2);

  //get new rules.
  startVector = rulz.r;
  distBetweenPoints = rulz.g;
  angleBetweenPoints = rulz.b;

  if (rand () % 2 == 0)
    drawDirection = 1;
  else
    drawDirection = -1;

  firstPoint = true;
  //other functions assume you have 2 lines
  getNewLine (dim, true);
}

void LinePattern :: render (SDL_Surface * dest, SDL_Rect dim)
{
  //draw each point on the SDL_Surface dest.

  //as a circle. You can use the w parameter to mark them as 'drawn' and 'not drawn.'

  //draw the current line.
  vector <SDL_Rect> :: iterator iter;

  Uint8 r;
  Uint8 g;
  Uint8 b;
  Uint8 a;

  if (points.size () > 1)
    {
      aalineRGBA(dest, points.at(0).x + dim.x, points.at(0).y  + dim.y, points.at(1).x + dim.x, points.at(1).y + dim.y, 0, 0, 0, 255);

      for (Uint8 i = points.size () - 2; i >= 1; i --)
	{
	  if (i == 1)
	    {r = 255; g = 255; b = 255; }
	  else if (i == 2)
	    {r = 142; g = 142; b = 212; }
	  else
	    {r = 57; g = 57; b = 168;}
	  a = 255;
	  
	  filledTrigonRGBA(dest, 
			   points.at(i+1).x + dim.x, points.at(i+1).y + dim.y, 
			   points.at(i).x + dim.x + (Sint16)(sin1 * points.at(i).w), points.at(i).y  + dim.y + (Sint16)(cos1 * points.at(i).h), 
			   points.at(i).x + dim.x + (Sint16)(sin2 * points.at(i).w), points.at(i).y + dim.y + (Sint16)(cos2 * points.at(i).h), 
			   r, g, b, a);

	  aatrigonRGBA(dest, 
		       points.at(i+1).x + dim.x, points.at(i+1).y + dim.y, 
		       points.at(i).x + dim.x + (Sint16)(sin1 * points.at(i).w), points.at(i).y  + dim.y + (Sint16)(cos1 * points.at(i).h), 
		       points.at(i).x + dim.x + (Sint16)(sin2 * points.at(i).w), points.at(i).y + dim.y + (Sint16)(cos2 * points.at(i).h), 
		       r, g, b, a);
	}
      
      for (Uint8 i = points.size () - 1; i > 0; i --)
	{
	  if (i == 0)
	    {r = 255; g = 255; b = 255; a = 64;}
	  else if (i == 1)
	    {r = 128; g = 0; b = 0; a = 255;}
	  else
	    {r = 255; g = 255; b = 255; a = i * (-64) + 330;}
	  
	  filledCircleRGBA(dest, points.at (i).x + dim.x, points.at (i).y + dim.y, points.at (i).w, r,g,b,a);
	}

      aacircleRGBA(dest, points.at(1).x + dim.x, points.at(1).y + dim.y,
		   points.at(1).w, 64, 64, 64, 255);
    }
  else
    {
      filledCircleRGBA(dest, points.at (0).x + dim.x, points.at (0).y + dim.y, points.at (0).w, 128,0,0,255);
      aacircleRGBA(dest, points.at(0).x + dim.x, points.at(0).y + dim.y,
		   points.at(0).w, 192, 192, 192, 255);
    }
}

int LinePattern :: getLastPoint (SDL_Rect & ptA)
{
  //returns the number of points in the vector.
  //throws an error if there are 0 points in the vector.

  ptA = points.back ();
  return points.size ();
}

int LinePattern :: getContactPoint (SDL_Rect & ptA)
{
  //sets ptA to the point the user should draw to.
  //returns the size of the vector.
  //throws an error if there are 0 points in the vector.

  if (points.size () >= 2)
    ptA = points.at (1);
  else
    ptA = points.at (0);
  return points.size ();
}

int LinePattern :: get2ndContactPoint (SDL_Rect & ptA)
{
  //sets ptA to the point the user should draw to.
  //returns the size of the vector.
  //throws an error if there are less than 3 points in the vector.

  ptA = points.at (2);
  return points.size ();
}

void LinePattern :: getNewLine (SDL_Rect dim, bool newline)
{
  //creates a new point.
  SDL_Rect newPt = generateNextPoint (dim);
  vector <SDL_Rect> :: iterator iter;

  //do starter stuff
  if (points.size () <= 0 && newline)
    {
      newPt.w = 8;
      newPt.h = 5;
      
      points.push_back (newPt);
    }
  else if (points.size () <= 1 && !newline)
    {
      //shrink the first point
      points.at (0).w = 1;
      points.at (0).h = 1;

      //add the newest point as the next one to click
      newPt.w = 8;
      newPt.h = 5;
      points.push_back (newPt);

      //add the rest
      for (int i = 0; i < 3; i++)
	{
	  newPt = generateNextPoint (dim);

	  newPt.w = 5 - i;
	  newPt.h = 3;

	  //add to queue
	  points.push_back (newPt);
	}	
    }
  else
    {
      //shrink the 2nd point
      points.at (1).w = 1;
      points.at (1).h = 1;

      //embiggen the 3rd point
      points.at (2).w = 8;
      points.at (2).h = 5;      

      iter = points.begin () + 2;
      while(iter < points.end()) {
	iter -> w ++;
	iter ++;
      }

      //pop the front
      points.erase (points.begin (), (points.begin () + 1) );

      //add to the back
      newPt.w = 3;
      newPt.h = 3;
      points.push_back (newPt);
    }
}

SDL_Rect LinePattern :: generateNextPoint (SDL_Rect dim)
{
  SDL_Rect newPt;
  SDL_Rect lastPt;

  Sint32 dir;
  Uint16 dist;
  Uint8 angl;
  Uint8 d2;
  Uint8 rnda;

  //start in the lower left corner and rotate clockwise. 8 is the center.
  if (firstPoint)
    {
      newPt.x = rand () % dim.w / 3;
      newPt.y = rand () % dim.h / 3;

      switch (startVector){
      case 0: newPt.y += 2 * dim.h / 3; break;
      case 1: newPt.y += dim.h / 3; break;
      case 2: break;
      case 3: newPt.x += dim.w / 3; break;
      case 4: newPt.x += 2 * dim.w / 3; break;
      case 5: newPt.x += 2 * dim.w / 3; newPt.y += dim.h / 3; break;
      case 6: newPt.x += 2 * dim.w / 3; newPt.y += 2 * dim.h / 3; break;
      case 7: newPt.x += dim.w / 3; newPt.y += 2 * dim.h / 3; break;
      case 8:
      default: newPt.x += dim.w / 3; newPt.y += dim.h / 3; break;
      }
      currentAngle = rand () % 360;
      firstPoint = false;
    }
  else
    {
      //generate new pt according to rules
      //get info on last points
      getLastPoint (lastPt);

      newPt.x = lastPt.x;
      newPt.y = lastPt.y;

      dir = currentAngle;

      //vary angle according to angleBetweenPoints and drawDirection.

      angl = angleBetweenPoints;
      rnda = rand () % 20;

      switch (angleBetweenPoints){
      case 0: 
	if (rnda < 1)
	  angl = 3;
	else if (rnda < 2)
	  angl = 2;
	else if (rnda < 5)
	  angl = 1;
	else
	  angl = 0;
	break;
      case 1: 
	if (rnda < 1)
	  angl = 3;
	else if (rnda < 2)
	  angl = 2;
	else if (rnda < 5)
	  angl = 0;
	else
	  angl = 1;
	break;
      case 2: 
	if (rnda < 1)
	  angl = 0;
	else if (rnda < 2)
	  angl = 1;
	else if (rnda < 5)
	  angl = 3;
	else
	  angl = 2;
	break;
      case 3: 
      default: 
	if (rnda < 1)
	  angl = 0;
	else if (rnda < 2)
	  angl = 1;
	else if (rnda < 5)
	  angl = 2;
	else
	  angl = 3;
	break;
      }

      //every 5 points or so, reverse the drawDirection.
      //randomize for extra fun!!!

      d2 = distBetweenPoints;

      if (rand () % 100 < 12)
	{
	  dir *= -1;
	  angl = rand () % 4;
	  d2 = rand () % 4;
	  drawDirection *= -1;
	}

      switch (angl){
      case 0: dir += drawDirection * (rand () % 20); break;
      case 1: dir += drawDirection * (20 + (rand () % 20)); break;
      case 2: dir += drawDirection * (40 + (rand () % 30)); break;
      case 3: 
      default: dir += drawDirection * (70 + (rand () % 20)); break;
      }

      //vary distance according to distBetweenPoints
      switch (d2){
      case 0: dist = 60; break;
      case 1: dist = 75; break;
      case 2: dist = 80; break;
      case 3: 
      default: dist = 84; break;
      }

      //clip dir
      while (dir < 0 || dir >= 360)
	{
	  if (dir < 0)
	    dir += 360;
	  if (dir >= 360)
	    dir -= 360;
	}

      //dir is in degrees (360)
      newPt.x += (int) ((double)dist * sin ((double)dir * 3.14159 / 180.0));
      newPt.y += (int) ((double)dist * cos ((double)dir * 3.14159 / 180.0));

      //if it's out of bounds, reverse and clip the next point in the boundary.
      //rebound and clipping: reverse the angle, too.

      if (newPt.x < 0)
	{
	  newPt.x = 0 - newPt.x;

	  if (dir > 180 && dir < 270)
	    dir = 90 + (dir - 180);
	  
	  if (dir >= 270 && dir < 360)
	    dir = 90 + (dir - 360);
	}
      if (newPt.x > dim.w)
	{
	  newPt.x = dim.w - newPt.x + dim.w;

	  if (dir >= 0 && dir < 90)
	    dir = 270 + (dir - 0);
	  
	  if (dir >= 90 && dir < 180)
	    dir = 270 + (dir - 180);
	}
      if (newPt.y < 0)
	{
	  newPt.y = 0 - newPt.y;

	  if (dir > 90 && dir < 180)
	    dir = 0 + (dir - 90);
	  
	  if (dir >= 180 && dir < 270)
	    dir = 0 + (dir - 270);
	}
      if (newPt.y > dim.h)
	{
	  newPt.y = dim.h - newPt.y + dim.h;

	  if (dir >= 0 && dir < 90)
	    dir = 180 + (dir - 90);
	  
	  if (dir > 270 && dir <= 360)
	    dir = 180 + (dir - 270);
	}

      //make sure they aren't the same point;
      while (lastPt.x == newPt.x && lastPt.y == newPt.y)
	{
	  //uhh, randomize?
	  lastPt.x = rand () % dim.w;
	  lastPt.y = rand () % dim.h;
	}

      //clipping
      if (newPt.x < 0)
	  newPt.x = 0;
      if (newPt.x > dim.w)
	  newPt.x = dim.w;
      if (newPt.y < 0)
	  newPt.y = 0;
      if (newPt.y > dim.h)
	  newPt.y = dim.h;

      //clip dir
      while (dir < 0 || dir >= 360)
	{
	  if (dir < 0)
	    dir += 360;
	  if (dir >= 360)
	    dir -= 360;
	}

      //update current angle
      currentAngle = dir;
    }
  return newPt;
}

Uint32 LinePattern :: chooseColor (SDL_Rect pt, SDL_Surface * dest, Uint8 & r, Uint8 & g, Uint8 & b, Uint8 & a)
{
  //picks the color, based on pt's h parameter.
  Uint32 ans;

  //do math magic here; assumed to be 5 dots long
  r = 192;
  g = 192;
  b = 192;
  a = 255 * pt.h / 5;

  ans = SDL_MapRGBA(dest -> format, r, g, b, a);
  return ans;
}

LinePattern :: ~LinePattern ()
{
}
