#include "SDL.h"
#include "SDL_framerate.h"
#include "SDL_ttf.h"

#include <stdlib.h>
#include <time.h>

#include <iostream>
#include <vector>
#include <string>
using namespace std;

#include "InputObj.h"
#include "UserInput.h"
#include "CpuInput.h"
#include "Battle.h"
#include "Combatant.h"

SDL_Surface * screen;
TTF_Font * font;
int screenResX = 640, screenResY = 480;

SDL_Event mainEvent;
FPSmanager fpsKeeper;
int fpsGoal = 30;

Battle battul;
UserInput masterInput;
CpuInput oppInput;
Combatant * lComb = NULL;
Combatant * rComb = NULL;
SDL_Color rulz;

bool escKeyHit = false;
bool resetGameHit = false;
int gameMode = 0;

const char title [] = "SwordDrawBattle";
const char icon [] = "SwordDrawBattle";

bool runInDebugMode = false;

void resetGame ()
{
  //setup screen
  if (lComb != NULL)
    delete lComb;

  lComb = new Combatant ();
  lComb -> setInput (&masterInput);
  lComb -> setName (* (new string ("Player")));
  lComb -> setMaxHP (30);
  
  battul.reset ();

  gameMode = 1; //set to battle mode
  resetGameHit = false;
}

Sint8 render ()
{
  //draw stuff on screen

  //lock the framerate.
  SDL_framerateDelay(&fpsKeeper);

  //once it has delayed, draw to the screen and flip it.
  SDL_FillRect(screen, NULL, SDL_MapRGB(screen -> format, 0, 0, 0));

  //call Battle to render this...if in BattleMode.
  if (gameMode == 1)
    battul.render (screen, font);

  //copy main window to screen.
  SDL_BlitSurface(screen, NULL, screen, NULL);

  //finally, flip the screen
  if (SDL_Flip(screen) == -1)
    {
      printf("Cannot flip screens: %s\n", SDL_GetError ());
      return -1;
    }

  return 0;
}

void setUpNewBadguy (Combatant * k, CpuInput * cpuin, SDL_Color & rulz)
{
  Uint32 numVic = battul.getNumberOfVictories ();
  Uint32 picker, hpPick = 0, damPick = 0, angPick = 0, distPick = 0, endPick = 0;
  Uint8 hpRank = 0, damRank = 0, angRank = 0, distRank = 0, endRank = 0;

  k -> setInput (cpuin);
  cpuin -> setStats (2);
  k -> setSide (false);
  k -> setStats (8, *(new string ("Bad dude")));

  //name should vary by enemy type
  rulz.r = rand () % 9;
  rulz.g = 0;
  rulz.b = 0;
  rulz.unused = 0;

  for (Uint32 index = 0; index < numVic; index ++)
    {
      //increase parameters.
      picker = rand () % 20;
      if (picker < 8)
	{
	  k -> addMaxHP (2);
	  hpPick ++;
	}
      else if (picker < 11)
	{
	  cpuin -> addDamage (1);
	  damPick ++;
	}
      else if (picker < 14)
	{
	  if (rulz.g < 3)
	    rulz.g += 1;
	  else
	    {
	      k -> addMaxHP (1);
	    }
	  distPick ++;
	}
      else if (picker < 17)
	{
	  if (rulz.b < 3)
	    rulz.b += 1;
	  else
	    {
	      k -> addMaxHP (1);
	    }
	  angPick ++;
	}
      else
	{
	  if (rulz.unused < 3)
	    rulz.unused += 1;
	  else
	    {	     
	      k -> addMaxHP (1);
	    }
	  endPick ++;
	}
    }

  //determine enemy name and animation by number of stat boosts.
  //normalize numbers by multiplying to LCD (24), eliminating probability.
  hpPick *= 3; 
  damPick *= 8; 
  angPick *= 8;
  distPick *= 8; 
  endPick *= 8;

  //OK, time for the massive tree. We're looking for the two highest elements.
  //Also, should pick higher-level names based on numVic.
  if (endPick >= hpPick)
    endRank ++;
  if (endPick >= damPick)
    endRank++;
  if (endPick >= angPick)
    endRank++;
  if (endPick >= distPick)
    endRank++;

  if (distPick >= angPick)
    distRank++;
  if (distPick >= endPick)
    distRank++;
  if (distPick >= hpPick)
    distRank++;
  if (distPick >= damPick)
    distRank++;

  if (angPick >= damPick)
    angRank++;
  if (angPick >= hpPick)
    angRank++;
  if (angPick >= endPick)
    angRank++;
  if (angPick >= distPick)
    angRank++;

  if (hpPick >= damPick)
    hpRank++;
  if (hpPick >= angPick)
    hpRank++;
  if (hpPick >= endPick)
    hpRank++;
  if (hpPick >= distPick)
    hpRank++;

  if (damPick >= hpPick)
    damRank++;
  if (damPick >= distPick)
    damRank++;
  if (damPick >= endPick)
    damRank++;
  if (damPick >= angPick)
    damRank++;

  if (angRank >= 3 && hpRank >= 3 && (angRank == 4 || hpRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Junior Longarms")));
      else if (numVic < 10)
	k -> setName (* (new string ("Longarms")));
      else if (numVic < 15)
	k -> setName (* (new string ("Grandpa Longarms")));
      else
	k -> setName (* (new string ("King Longarms")));
    }

  if (angRank >= 3 && damRank >= 3 && (angRank == 4 || damRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Bandit")));
      else if (numVic < 10)
	k -> setName (* (new string ("Shadow")));
      else if (numVic < 15)
	k -> setName (* (new string ("Assassin")));
      else
	k -> setName (* (new string ("The CEO")));
    }

  if (distRank >= 3 && damRank >= 3 && (distRank == 4 || damRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Brute")));
      else if (numVic < 10)
	k -> setName (* (new string ("Giant")));
      else if (numVic < 15)
	k -> setName (* (new string ("McBamf")));
      else
	k -> setName (* (new string ("Hammertime")));
    }


  if (endRank >= 3 && hpRank >= 3 && (endRank == 4 || hpRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Crystal")));
      else if (numVic < 10)
	k -> setName (* (new string ("Spire")));
      else if (numVic < 15)
	k -> setName (* (new string ("Obelisk")));
      else
	k -> setName (* (new string ("Big Shiny")));
    }

  if (endRank >= 3 && angRank >= 3 && (endRank == 4 || angRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Fairy")));
      else if (numVic < 10)
	k -> setName (* (new string ("Sprite")));
      else if (numVic < 15)
	k -> setName (* (new string ("Titania")));
      else
	k -> setName (* (new string ("Pretty Pretty Princess!")));
    }

  if (distRank >= 3 && angRank >= 3 && (distRank == 4 || angRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Mosquitoes")));
      else if (numVic < 10)
	k -> setName (* (new string ("Killer Beez")));
      else if (numVic < 15)
	k -> setName (* (new string ("Locusts")));
      else
	k -> setName (* (new string ("It Tickles...")));
    }

  if (hpRank >= 3 && damRank >= 3 && (hpRank == 4 || damRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Golem")));
      else if (numVic < 10)
	k -> setName (* (new string ("Guardian")));
      else if (numVic < 15)
	k -> setName (* (new string ("WarMech")));
      else
	k -> setName (* (new string ("Killbot2000")));
    }
  
  if (endRank >= 3 && damRank >= 3 && (endRank == 4 || damRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Apprentice")));
      else if (numVic < 10)
	k -> setName (* (new string ("Wizard")));
      else if (numVic < 15)
	k -> setName (* (new string ("Archmage")));
      else
	k -> setName (* (new string ("Zappity-Doom")));
    }
  
  if (endRank >= 3 && distRank >= 3 && (endRank == 4 || distRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Baby Dragon")));
      else if (numVic < 10)
	k -> setName (* (new string ("Wyrm")));
      else if (numVic < 15)
	k -> setName (* (new string ("Red Dragon")));
      else
	k -> setName (* (new string ("Teh Ancient One!")));
    }

  if (distRank >= 3 && hpRank >= 3 && (distRank == 4 || hpRank == 4))
    {
      if (numVic < 5)
	k -> setName (* (new string ("Sludge")));
      else if (numVic < 10)
	k -> setName (* (new string ("Goo")));
      else if (numVic < 15)
	k -> setName (* (new string ("Slime")));
      else
	k -> setName (* (new string ("Cubicle")));
    }
}

void addNewBadGuy ()
{
  delete rComb;
  rComb = new Combatant ();
  
  //use settings to load a new enemy
  setUpNewBadguy (rComb, &oppInput, rulz);

  battul.init (lComb, rComb, rulz);
}

Sint8 gameLoop()
{
  bool done = false;

  while (!done)
    {
      //get events until the queue is empty
      while (SDL_PollEvent(&mainEvent) == 1)
	{
	  //for each event:
	  switch(mainEvent.type){
	  case SDL_KEYDOWN:
	    //check to see if it wants to quit	    
	    if(mainEvent.key.keysym.sym==SDLK_ESCAPE)
	      escKeyHit = true;
	    break;
	  case SDL_KEYUP:
	    //check to see if it wants to quit	    
	    if(mainEvent.key.keysym.sym==SDLK_ESCAPE)
	      escKeyHit = false;
	    if(mainEvent.key.keysym.sym==SDLK_F10)
	      resetGameHit = true;
	    break;
	  case SDL_APPMOUSEFOCUS:
	    if (mainEvent.active.gain == 1)
	      masterInput.forceMousePos ();
	    break;
	  default:
	    break;
	  }

	  if (gameMode == 0)
	    masterInput.processInput (mainEvent);
	  else if (gameMode == 1)
	    battul.processInput (mainEvent);
	}
      //done with all events

      //execute pending input decisions here
      //tell battle to do stuff here
      battul.gameLoop ();

      //this is used for main.
      //check to see if it wants to quit
      if(escKeyHit)
	done = true;
      
      //make sure this is only when in Battul Mode.

      //put reset key here
      if (resetGameHit)
	resetGame ();

      if (gameMode == 1 && battul.getStatus () == 0)
	{
	  addNewBadGuy ();
	}

      //perform results of logic here

      //draw the screen
      if (render () == -1)
	return -1;
    }
  return 0;
}

Sint8 loadRound()
{
  //set up the rules.
  return 0;
}

Sint8 initGameVariables()
{
  //set the framerate
  SDL_initFramerate(&fpsKeeper);
  SDL_setFramerate(&fpsKeeper, fpsGoal);

  //init keyboard
  SDL_EnableKeyRepeat(1, 30);

  //set the randomizer seed.
  srand( time(NULL) );
  return 0;
}

Sint8 initGame()
{
  if (initGameVariables() == -1)
    return -1;

  if (loadRound () == -1)
    return -1;

  return 0;
}

Sint8 initGfx(bool debugMode)
{
  //set up screen
  if (debugMode)
    screen = SDL_SetVideoMode(640, 480, 32, SDL_HWSURFACE|SDL_DOUBLEBUF);
  else
    screen = SDL_SetVideoMode(640, 480, 32, SDL_HWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);

  if (screen == NULL)
    {
      printf ("Error initializing video, Attempting software surface.");
      if (debugMode)
	screen = SDL_SetVideoMode(screenResX, screenResY, 32, SDL_SWSURFACE|SDL_DOUBLEBUF);
      else
	screen = SDL_SetVideoMode(screenResX, screenResY, 32, SDL_SWSURFACE|SDL_DOUBLEBUF|SDL_FULLSCREEN);
    }

  if (screen == NULL)
    {
      printf ("Error initializing video: %s", SDL_GetError ());
      return -1;
    }

  //reset game
  resetGame ();

  //disbale mouse
  SDL_ShowCursor(SDL_DISABLE);
  return 0;
}

Sint8 initSDL(bool debugMode)
{
  if (SDL_Init(SDL_INIT_VIDEO) == Sint8(-1))
    {
      printf ("Unable to init SDL: %s\n", SDL_GetError());
      return -1;
    }

  if(TTF_Init()==-1) 
    {
      printf("Unable to init SDL_TTF: %s\n", TTF_GetError());
      return -1;
    }

  font=TTF_OpenFont("text/FreeSans.ttf", 12);
  
  if(font == NULL) 
    {
      printf("TTF_OpenFont: %s\n", TTF_GetError());
      return -1;
    }
  return 0;
}

Sint8 initEngine()
{
  if (initSDL(runInDebugMode) == -1)
    return -1;

  if (initGfx (runInDebugMode) == -1)
    return -1;

  //window futzing
  SDL_WM_SetCaption(title, icon);

  return 0;
}

int main (int argc, char *argv[])
{
  if (initEngine() == -1)
    return -1;

  if (initGame () == -1)
    return -1;

  if (gameLoop () == -1)
    return -1;

  SDL_ShowCursor(SDL_ENABLE);
  SDL_FreeSurface(screen);
  SDL_Quit ();
  TTF_Quit ();
  return 0;
};
